/** Some templates I used while developing my rtti module that are not
 *  specific to the problem that module is trying to solve.
 *  
 *  Authors:
 *  Frits van Bommel
 */
module util;

/** Returns a constant array of T, initialized by the Ts parameters.
 */
template Array(T, Ts...) {
        alias ArrayImpl!(T, Ts).Array Array;
}

private template ArrayImpl(T, Ts...) {
    T[(Ts.length)] Array; // can't be const: "Şmissing initializer" :(
    
    private static this() {
        foreach(idx, elt; Ts) {
            Array[idx] = elt;
        }
    }
}

/// Returns a tuple consisting of the base class and interfaces of T.
template BaseTypeTuple(T) {
    static if (is(T Bases == super))
        alias Bases BaseTypeTuple;
    else
        static assert("Not a valid argument: " ~ T.mangleof);
}

/** Returns Value rounded up to a multiple of Alignment, which must be a power
 *  of two.
 */
template alignUp(size_t Alignment, size_t Value) {
    static assert((Alignment & (Alignment - 1)) == 0);     // al must be power of two
    const alignUp = (Value + Alignment - 1) & ~(Alignment - 1);
}

/// Converts a type tuple to a string.
template TypeTupleToString(V...) {
    static if (V.length > 0)
        const TypeTupleToString = V[0].mangleof ~ " " ~ TypeTupleToString!(V[1..$]);
    else
        const char[] TypeTupleToString = "";
}

/// Returns true if T is an associative array.
template isAssociativeArray(T) {
    static if (is(typeof(T.keys)) && is(typeof(T.values))) {
        const bool isAssociativeArray =
            is(T == typeof(T.values[0])[typeof(T.keys[0])]);
    } else {
        const bool isAssociativeArray = false;
    }
}
